
function Chapter_3_Figure_7_regulated_system
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This matlab file reproduces simulation studies in the book:
% Systems Biology: Mathematical modeling and model analysis
% by A. Kremling
%
% It is related to Figure 7 (right) in Chapter 3
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all

disp(' ')
disp(' NOTE: The simulation need some time!')
disp('       Therefore the time during the simulation of')
disp('       the CME as well as the number of cells that')
disp('       are simulated for the stochasitic version are shown.')

disp('   ')
disp(' Press ENTER to start!  ')
pause

% --- parameters

%central_numbers
V = 2e-15;                  % volumen
N = [-1  1 0 0 0            % stoichiometry
      1 -1 0 0 0
      0  0 1 -1 1];
  
L = (abs(N)-N)/2;           % regulation
L(2,3) = 1;
L(3,1) = 2;

n0 = [2;1;100];             % AB
tmax = 1000;                % t maximum
c(1) = 1e-4/6;              % c - values
c(2) = 1;
c(3) = 3.5;
c(4) = 0.01;
c(5) = 0.2;
c=c';
max_cell = 1000;            % number of simulated cells
pMax = 1200;                % maximum number of  protein molecules
mMax = n0(1)+n0(2);         % maximum number promotor molecules
fig_nr=1;                   % figure for output

p.c = c;

%______________________________________________________________________________
% --- calculation: Master Equation %%%%%%%%%%%%%%%%%%%%%%%%

% Initial condition of the Master Equation: 

P0 = zeros(mMax+1,pMax+1);
P0(n0(2)+1,n0(3)+1) = 1; 

%P0 = ones(mMax+1,pMax+1)/((mMax+1)*(pMax+1));
%P0 = reshape(P0,(mMax+1)*(pMax+1),1);
tSim = [0:1:tmax];

tic
[tCME, xCME] = ode15s(@ttMasterequation_b,tSim,P0,[],p,mMax, pMax);  
toc

P = reshape(xCME(end, :), mMax + 1, pMax + 1); 
% the probabilities P(n_M, n_p, t) are saved in vektor
% xCME; with reshape the vektor is transformed in a matrix 
 

% --- plotting

figure(fig_nr+1),hold on
plot(0:pMax,P(1,:)+P(2,:)+P(3,:)+P(4,:),'r');
master_right=P(1,:)+P(2,:)+P(3,:)+P(4,:);

for l = 1: length(tCME),
    PProt(l) = 0;
    P = reshape(xCME(l, :), mMax + 1, pMax + 1);
    for k = 1:pMax+1,
        for kk = 1:mMax+1,
            PProt(l) = PProt(l) + (k-1)*P(kk,k);
        end
    end

end

figure(fig_nr),subplot(223)
plot(tCME,PProt)
%______________________________________________________________________________
% --- calculation: Gillespie algorithm %%%%%%%%%%%%%%%%%%%%%%%%


disp(' ')
disp(' Stochastic simulation starts now!' )
disp(' ')
disp(' ')

rand('state',sum(100*clock));

h = zeros(length(c),1);

Dz = []; Do = []; PP = [];

for k = 1:max_cell,
  disp(k)
  T = [0]; Nx = [n0'];
  [t,n] = deal(0,n0);
  
while t <= tmax

  for l = 1:length(c-1),    % determine reaction partner for
                            % reaction 1-4 (reaction 5 is a special case)
  
    z = find(L(:,l) ~= 0);  
    g  = [];
    
    for r = 1:length(z),
      if (n(z(r)) >=  L(z(r),l))
        g(r) =  nchoosek(n(z(r)),L(z(r),l));
      else
        g(r) = 0;
      end
    end
    h(l) = prod(g);
  end
  
    a = c.*h;    
    a(5) = c(5);          % special case reaction 5
    
    astr = sum(a);
    if ~astr, disp('astr = 0'), break, end
    tau = -1/astr*log(rand);
    u = find(cumsum(a)>astr*rand,1);
    n = n + N(:,u);       % update by stoichiometry
    t = t + tau;          % update time
    
    T = [T;t];Nx = [Nx;n'];   % data backup
   
end

% --- plotting

figure(fig_nr)
subplot(221),plot(T,Nx(:,1)),axis([0 tmax 0 mMax]),hold on
subplot(222),plot(T,Nx(:,2)),axis([0 tmax 0 mMax]),hold on
subplot(223),plot(T,Nx(:,3)),axis([0 tmax 0 pMax]),hold on

values_end(k,:) = [T(end) Nx(end,:)];
[time_des,states_des] =  make_vec(T,Nx,tmax);      % average

Dz = [Dz  states_des(:,1)]; 
Do = [Do  states_des(:,2)]; 
PP = [PP  states_des(:,3)];

end

%______________________________________________________________________________
% --- calculation: Deterministic Simulation %%%%%%%%%%%%%%%%%%%%  

NaV = 6.022e23*V;
[tt,y] = sim_dyn_gen_work(N,L,c,tmax,n0,V);
dz = y(:,1)*NaV;
do = y(:,2)*NaV;
prot = y(:,3)*NaV;

 % --- plotting
 
 figure(fig_nr);
 subplot(221),plot(time_des,mean(Dz'),'g','Linewidth',2);hold on
 subplot(222),plot(time_des,mean(Do'),'g','Linewidth',2);hold on
 subplot(223),plot(time_des,mean(PP'),'g','Linewidth',2);hold on

 subplot(221),plot(tt,dz,'r','Linewidth',2)
 subplot(222),plot(tt,do,'r','Linewidth',2)
 subplot(223),plot(tt,prot,'r','Linewidth',2)


NN_right = hist(values_end(:,4),1:pMax);
figure(fig_nr+1), hold on
bar(NN_right/max_cell)
axis([-10 pMax 0 0.035])
save results_figure_right_regulated_system



%_____________________________________________________________
%  functions
%_____________________________________________________________
%_____________________________________________________________
% deterministic System

function [t,y] = sim_dyn_gen_work(N,L,c,tmax,y00,V)

NaV = 6.022e23*V;
K = sum(L)';
p.k = c.*((NaV).^(K-1))./(prod(factorial(L))');
p.N = N;
p.L = L;
y0 = y00/NaV;
time = 0:tmax/1000:tmax;

options = odeset('RelTol',1e-6,'AbsTol',1e-9,'Vectorized','on');
[t,y] = ode15s(@f,time,y0,options,p);

% ----------------------------------------------------------

function out = f(t,y,p)

p.N = p.N(:,1:4);
r1 = p.k(1)*y(1,:).*y(3,:).^2;
r2 = p.k(2)*y(2,:);
r3 = p.k(5)*ones(1,length(y(2,:)))+p.k(3)*y(2,:);
r4 = p.k(4)*y(3,:);
rr = [r1;r2;r3;r4];

out = p.N*rr;


%_____________________________________________________________________________

function [time_des,states_des] =  make_vec(t,states,tmax)
lt=length(t);
[nstates,mstates] = size(states);

time_des = 0.2:0.2:tmax;
time_des = time_des';

for u = 1:length(time_des)
     
    for  r = 1:length(t),
         if t(r) <= time_des(u),
         
             for st = 1:mstates
                 states_des(u,st) = states(r,st);
             end
        end;
    end;

end


function dP = ttMasterequation_b(t,P,p,mMax,pMax)

t

P = reshape(P, (mMax + 1), (pMax + 1));

dP = zeros(mMax + 1, pMax + 1);

for nM = 0:mMax
    for nP = 0:pMax   

      if (nM == 0) && (nP == 0)                      %   0, 0
    
      a2 = p.c(2)*(nM+1);
      a4 = p.c(4)*(nP+1);
      a6 = p.c(1)*(mMax-nM)*nP^2;
      a8 = p.c(3)*nM+p.c(5);  
      a2P = a2*P(nM+1+1,nP+1);
      a4P = a4*P(nM+1,nP+1+1);
      a6P = a6*P(nM+1,nP+1);
      a8P = a8*P(nM+1,nP+1);
      dP(nM + 1, nP + 1) = a2P + a4P - a6P - a8P;
                       
      elseif (nM == 0) && (nP == pMax)                   % 0 , max
     
      
      a2 = p.c(2)*(nM+1);
      a3 = p.c(3)*nM+p.c(5);
      a6 = p.c(1)*(mMax-nM)*nP*(nP-1)/2;
      a7 = p.c(4)*nP;
      a2P = a2*P(nM+1+1,nP+1);
      a3P = a3*P(nM+1,nP-1+1);
      a6P = a6*P(nM+1,nP+1);
      a7P = a7*P(nM+1,nP+1);
     dP(nM + 1, nP + 1) =  a2P + a3P - a6P - a7P;
 
      elseif (nM == mMax) && (nP == 0)                 % max , 0
     
      a1 = p.c(1)*(mMax-nM+1)*nP^2;
      a4 = p.c(4)*(nP+1);
      a5 = p.c(2)*nM;
      a8 = p.c(3)*nM+p.c(5);
      a1P = a1*P(nM-1+1,nP+1);
      a4P = a4*P(nM+1,nP+1+1);
      a5P = a5*P(nM+1,nP+1);
      a8P = a8*P(nM+1,nP+1);
      dP(nM + 1, nP + 1) =  a1P +  a4P - a5P - a8P;	
      
      elseif (nM == mMax) && (nP == pMax)            % max , max
      
a1 = p.c(1)*(mMax-nM+1)*nP*(nP-1)/2;
      a3 = p.c(3)*nM+p.c(5);
      a5 = p.c(2)*nM;
      a7 = p.c(4)*nP;
      a1P = a1*P(nM-1+1,nP+1);
      a3P = a3*P(nM+1,nP-1+1);
      a5P = a5*P(nM+1,nP+1);
      a7P = a7*P(nM+1,nP+1);
  
      dP(nM + 1, nP + 1) =  a1P + a3P - a5P - a7P;	
      
      elseif (nM == 0) && (nP > 0) && (nP < pMax)           % 0, between

      a2 = p.c(2)*(nM+1);
      a3 = p.c(3)*nM+p.c(5);
      a4 = p.c(4)*(nP+1);
      a6 = p.c(1)*(mMax-nM)*nP*(nP-1)/2;
      a7 = p.c(4)*nP;
      a8 = p.c(3)*nM+p.c(5);
      a2P = a2*P(nM+1+1,nP+1);
      a3P = a3*P(nM+1,nP-1+1);
      a4P = a4*P(nM+1,nP+1+1);
      a6P = a6*P(nM+1,nP+1);
      a7P = a7*P(nM+1,nP+1);
      a8P = a8*P(nM+1,nP+1);
  	
      dP(nM + 1, nP + 1) = a2P + a3P + a4P - a6P -a7P - a8P;
               
      elseif (nM > 0) && (nM < mMax) && (nP == 0)             %  between , 0
  
a1 = p.c(1)*(mMax-nM+1)*nP^2;
      a2 = p.c(2)*(nM+1);
      a4 = p.c(4)*(nP+1);
      a5 = p.c(2)*nM;
      a6 = p.c(1)*(mMax-nM)*nP^2;
      a7 = p.c(4)*nP;
      a8 = p.c(3)*nM+p.c(5);
      a1P = a1*P(nM-1+1,nP+1);
      a2P = a2*P(nM+1+1,nP+1);
      a4P = a4*P(nM+1,nP+1+1);
      a5P = a5*P(nM+1,nP+1);
      a6P = a6*P(nM+1,nP+1);
      a7P = a7*P(nM+1,nP+1);
      a8P = a8*P(nM+1,nP+1);
  	
      dP(nM + 1, nP + 1) = a1P + a2P + a4P - a5P - a6P - a8P; 
                   
      elseif (nM == mMax) && (nP > 0) && (nP < pMax)       % max, between
                                                             
      
a1 = p.c(1)*(mMax-nM+1)*nP*(nP-1)/2;
      a3 = p.c(3)*nM+p.c(5);
      a4 = p.c(4)*(nP+1);
      a5 = p.c(2)*nM;
      a6 = p.c(1)*(mMax-nM)*nP*(nP-1)/2;
      a7 = p.c(4)*nP;
      a8 = p.c(3)*nM+p.c(5);
      a1P = a1*P(nM-1+1,nP+1);
      a3P = a3*P(nM+1,nP-1+1);
      a4P = a4*P(nM+1,nP+1+1);
      a5P = a5*P(nM+1,nP+1);
      a6P = a6*P(nM+1,nP+1);
      a7P = a7*P(nM+1,nP+1);
      a8P = a8*P(nM+1,nP+1);
  	
      dP(nM + 1, nP + 1) =   a1P + a3P + a4P - a5P - a7P - a8P;
 
      elseif (nM > 0) && (nM < mMax) && (nP == pMax)  % between , max
  a1 = p.c(1)*(mMax-nM+1)*nP*(nP-1)/2;
      a2 = p.c(2)*(nM+1);
      a3 = p.c(3)*nM+p.c(5);

      a5 = p.c(2)*nM;
      a6 = p.c(1)*(mMax-nM)*nP*(nP-1)/2;
      a7 = p.c(4)*nP;
      a8 = p.c(3)*nM+p.c(5);
      a1P = a1*P(nM-1+1,nP+1);
      a2P = a2*P(nM+1+1,nP+1);
      a3P = a3*P(nM+1,nP-1+1);

      a5P = a5*P(nM+1,nP+1);
      a6P = a6*P(nM+1,nP+1);
      a7P = a7*P(nM+1,nP+1);
      a8P = a8*P(nM+1,nP+1);	
      dP(nM + 1, nP + 1) = a1P + a2P + a3P - a5P -a6P - a7P;
            
      else    
      
a1 = p.c(1)*(mMax-nM+1)*nP*(nP-1)/2;
      a2 = p.c(2)*(nM+1);
      a3 = p.c(3)*nM+p.c(5);
      a4 = p.c(4)*(nP+1);
      a5 = p.c(2)*nM;
      a6 = p.c(1)*(mMax-nM)*nP*(nP-1)/2;
      a7 = p.c(4)*nP;
      a8 = p.c(3)*nM+p.c(5);
     a1P = a1*P(nM-1+1,nP+1);
      a2P = a2*P(nM+1+1,nP+1);
      a3P = a3*P(nM+1,nP-1+1);
      a4P = a4*P(nM+1,nP+1+1);
      a5P = a5*P(nM+1,nP+1);
      a6P = a6*P(nM+1,nP+1);
      a7P = a7*P(nM+1,nP+1);
      a8P = a8*P(nM+1,nP+1);	
        dP(nM + 1, nP + 1) =  a1P + a2P + a3P + a4P - a5P - a6P - a7P - a8P;
        end
    end
end

dP = reshape(dP, (mMax + 1)*(pMax + 1), 1);


