function Chapter_6_Figure_4

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This matlab file reproduces simulation studies in the book:
% Systems Biology: Mathematical modeling and model analysis
% by A. Kremling
%
% It is related to Figure 4 in Chapter 6
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


fo=optimset('LargeScale','off','MaxFunEvals',2000,'Display','off','TolX',1e-8,'TolFun',1e-8,'MaxIter',2000);

% --- Parameters

K   = 0.2;               % reaction-constant initiation of promotor
re0 = 1.32e-3;           % initial-concentration regulator
d0  = 1.32e-5;           % initial-concentration promotor
re02 = 1.32e-2;          % value for "image right"
d02  = 1.32e-4;          % value for "image right"
Kb  = 1.32e-6;           % reaction-constant for interaction with binding-position
Kb1 = 1.32e-4;           % value for "image left"
Ki  = 0.2;               % reaction-constant binding of inductor
ind = 0:0.01:2;          % concentration inducer
alpha = 2;               % cooperativity
beta  = 0.05;            % affinity
Alpha = [1 2 5];
Beta  = [0.09 .11 .2];
N     = [1 4];          % number of binding-positions at regulator

% --- calculation

% ---------- image left --------------- %
start1=[d0/10 re0/30];
for l=1:length(N)
    for i = 1:length(ind)
        n=N(l);
        p=[d0;re0; K; Kb1; Ki; ind(i);n];
        [x,fval]=fsolve(@sys_algebra3,start1,fo,p);
        d(i)  = x(1);   
        re(i) = x(2);   
        psi(i,l) = (d(i)/K)/d0; % transcription-efficiency
        start = [d(i) re(i)];
    end
end;

% ---------- image right --------------- %
start2=[d02/10 re02/30];
for l=1:length(Beta)
    for i = 1:length(ind)
        beta=Beta(l);
        p=[d02;re02; K; Kb; Ki; ind(i); alpha; beta];
        [x,fval]=fsolve(@sys_algebra2,start2,fo,p);
        d3(i)  = x(1);  % 
        re3(i) = x(2);  % 
        psi3(i,l) = (d3(i)/K)/d02; % transcription-efficiency
        start = [d3(i) re3(i)];
    end
end;

% ---------- image middle --------------- %
start1=[d0/10 re0/30];
for l=1:length(Alpha)
    for i = 1:length(ind)
        alpha=Alpha(l);
        p=[d0;re0; K; Kb; Ki; ind(i); alpha; beta];
        [x,fval]=fsolve(@sys_algebra2,start1,fo,p);
        d2(i)  = x(1);   
        re2(i) = x(2);   
        psi2(i,l) = (d2(i)/K)/d0; % transcription-efficiency
        start = [d2(i) re2(i)];
    end
end;

% --- plotting

figure(1)
subplot(131);plot(ind,psi(:,1),'k-.',ind,psi(:,2),'k');
xlabel('Inductor'); ylabel('transcription-efficiency'); 
text(0.5,0.5,' \rightarrow n'); 
title({'Transcription efficiency calculated using the first variation';...
    'of the model with Equation (6.36)'})

subplot(132);plot(ind,psi2(:,1),'-k',ind,psi2(:,2),'--k',ind,psi2(:,3),'-.k');
xlabel('Inductor');ylabel('transcription-efficieny');
text(0.5,0.5,' \leftarrow \alpha'); 
title({'Transcription efficiency calculated using'...
    'the second variation and Equation (6.39).'});

subplot(133);plot(ind,psi3(:,1),'-k',ind,psi3(:,2),'--k',ind,psi3(:,3),'-.k');
xlabel('Inductor');ylabel('transcription-efficiency');
text(0.5,0.3,' \rightarrow \beta'); 
title({'Transcription efficiency calculated using'...
    'the second variation and Equation (6.39).'});

% ---------------------------------------------------------
% ---------------------------------------------------------

function yp = sys_algebra2(y,p)

% ----------------------------- 

d0     =  p(1);
re0    =  p(2);
K      =  p(3);
Kb     =  p(4); 
Ki     =  p(5);
ind    =  p(6);
alpha  =  p(7);
beta   =  p(8);

% ----------------------------- 
d  = y(1); 
re = y(2);
% ----------------------------- 
yp = zeros(2,1);

rei1 = 4*re*ind/Ki;
rei2 = 6*alpha*re*ind^2/Ki^2;
rei3 = 4*alpha^3*re*ind^3/Ki^3;
rei4 = alpha^6*re*ind^4/Ki^4;

rd1 = beta * rei1 * d/Kb;
rd2 = beta^2 * rei2 * d/Kb;
rd3 = beta^3 * rei3 * d/Kb;

yp(1) = d0 - ( d * (1+1/K+re/Kb) + rd1 + rd2 + rd3);

yp(2) = re0 - re - re*d/Kb - rei1 - rei2 - rei3 - rei4 - rd1 - rd2 - rd3;


% ---------------------------------------------------------
% ---------------------------------------------------------

function yp = sys_algebra3(y,p)

% ----------------------------- 

d0     =  p(1);
re0    =  p(2);
K      =  p(3);
Kb     =  p(4); 
Ki     =  p(5);
ind    =  p(6);
n      =  p(7);

% ----------------------------- 
d  = y(1); 
re = y(2);
% ----------------------------- 
yp = zeros(2,1);

yp(1)=d0-d*(1+1/K+re/Kb);
yp(2)=re0-re*(1+d/K+(ind^n)/Ki);


% ---------------------------------------------------------
% ---------------------------------------------------------

function ze = sys(y,p)

% --- parameter definition 

d10 =  p(1);
p0  =  p(2);
d20 =  p(3);
c0  =  p(4);
K1  =  p(5);
K2  =  p(6);
al  =  p(7);

% --- definition of algebraic states

p  = y(1);
c  = y(2);

% --- model-equations

ze = zeros(2,1);

d1 = d10/(1+p/K1);
d2 = d20/(1+c/K2+p/K1+al*p*c/(K1*K2));

ze(1) = p0 - p -  d1*p/K1 -  d2*p/K1 -  al*d2*p*c/(K1*K2);

ze(2) = c0 - c -  d2*c/K2 -  al*d2*p*c/(K1*K2);


