function Chapter_6_Figure_7

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This matlab file reproduces simulation studies in the book:
% Systems Biology: Mathematical modeling and model analysis
% by A. Kremling
%
% It is related to Figure 7 in Chapter 6
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;
% --- parameter

D10 = 1.3e-4;
P0 = 2.0e-3;
D20 = 1.3e-5;
C0 = 1.3e-3;

K1 =  1.0e-3; 
K2 =  1.0e-3;
alpha = 100.0;

% --- calculation

% --- loop over C with constant P
C0 = 1.0e-5:1e-5:2e-3;  
fo=optimset('LargeScale','off','MaxFunEvals',2000,'Display','off','TolX',1e-8,'TolFun',1e-8,'MaxIter',2000);

for i = 1:length(C0),

 start=[P0/30 C0(i)/30];
 para=[D10; P0; D20; C0(i); K1; K2; alpha];

% --- solving the equations
 
 [x,fval]=fsolve(@sys,start,fo,para);

 p  = x(1);
 c  = x(2);
 d1 = D10/(1+p/K1);
 d2 = D20/(1+c/K2+p/K1+alpha*p*c/(K1*K2));
 
% --- calculation of the transcription-efficiency

psi11(i) = (d1*p/K1)/D10; 
psi21(i) = (d2*p/K1)/D20 +  alpha*d2*p*c/(K1*K2)/D20;

end;

% ---------------------------------------------

% --- loop over P with constant C
P0 = 1e-5:1e-5:2e-3;  
C0 = 1.3e-3;

for i = 1:length(P0),
  
  start=[P0(i)/30 C0/30];
  para=[D10; P0(i); D20; C0; K1; K2; alpha];

 % --- solving the equations
  [x,fval]=fsolve(@sys,start,fo,para);

  p  = x(1);
  c  = x(2);
  d1 = D10/(1+p/K1);
  d2 = D20/(1+c/K2+p/K1+alpha*p*c/(K1*K2));

% --- calculation of the transcription-efficiency

 
 psi12(i) = (d1*p/K1)/D10; 
 psi22(i) = (d2*p/K1)/D20 +  alpha*d2*p*c/(K1*K2)/D20;

end;

% --- plotting

P0 = 1e-5:1e-5:2e-3;
C0 = 1.0e-5:1e-5:2e-3;  
figure;
subplot(121);plot(P0,psi12,'--k',P0,psi22,'-k'); 
title('Influence of P on binding-position with constant C');
xlabel('P0');ylabel('Transcription-efficiency');legend('D1','D2','location','east'); ylim([0 1])

subplot(122);plot(C0,psi11,'--k',C0,psi21,'-k');
title('Influence of C on both binding-positions with constant P');
xlabel('C0');ylabel('Transcription-efficiency');legend('D1','D2','location','east');ylim([0 1])

end

% ---------------------------------------------------------
% ---------------------------------------------------------

function yp = sys_algebra2(y,p)

% ----------------------------- 

d0     =  p(1);
re0    =  p(2);
K      =  p(3);
Kb     =  p(4); 
Ki     =  p(5);
ind    =  p(6);
alpha  =  p(7);
beta   =  p(8);

% ----------------------------- 
d  = y(1); 
re = y(2);
% ----------------------------- 
yp = zeros(2,1);

rei1 = 4*re*ind/Ki;
rei2 = 6*alpha*re*ind^2/Ki^2;
rei3 = 4*alpha^3*re*ind^3/Ki^3;
rei4 = alpha^6*re*ind^4/Ki^4;

rd1 = beta * rei1 * d/Kb;
rd2 = beta^2 * rei2 * d/Kb;
rd3 = beta^3 * rei3 * d/Kb;

yp(1) = d0 - ( d * (1+1/K+re/Kb) + rd1 + rd2 + rd3);

yp(2) = re0 - re - re*d/Kb - rei1 - rei2 - rei3 - rei4 - rd1 - rd2 - rd3;

end
% ---------------------------------------------------------
% ---------------------------------------------------------

function yp = sys_algebra3(y,p)

% ----------------------------- 

d0     =  p(1);
re0    =  p(2);
K      =  p(3);
Kb     =  p(4); 
Ki     =  p(5);
ind    =  p(6);
n      =  p(7);

% ----------------------------- 
d  = y(1); 
re = y(2);
% ----------------------------- 
yp = zeros(2,1);

yp(1)=d0-d*(1+1/K+re/Kb);
yp(2)=re0-re*(1+d/K+(ind^n)/Ki);


end
% ---------------------------------------------------------
% ---------------------------------------------------------

function ze = sys(y,p)

% --- paramterdefinition 

d10 =  p(1);
p0  =  p(2);
d20 =  p(3);
c0  =  p(4);
K1  =  p(5);
K2  =  p(6);
al  =  p(7);

% --- definition of algebraic states

p  = y(1);
c  = y(2);

% --- Model-equations

ze = zeros(2,1);

d1 = d10/(1+p/K1);
d2 = d20/(1+c/K2+p/K1+al*p*c/(K1*K2));

ze(1) = p0 - p -  d1*p/K1 -  d2*p/K1 -  al*d2*p*c/(K1*K2);

ze(2) = c0 - c -  d2*c/K2 -  al*d2*p*c/(K1*K2);
end
