function Chapter_8_Figure_14

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This matlab file reproduces simulation studies in the book:
% Systems Biology: Mathematical modeling and model analysis
% by A. Kremling
%
% It is related to Figure 14 in Chapter 8
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% --- parameters

% Stationary Values
X0 = [0.5 0.5]';

% The stoichiometric matrix
N = [1 -1 0
     0 2 -1];

% The null space of N
K = null(N,'r');

% The row vector of steady state fluxes
c = 2;
v = (K*c)';

% --- calculation and plotting

% Define the matrix LAMBDA
LAMBDAs=N.*v(ones(2,1),:);
LAMBDA=LAMBDAs./X0(:,ones(3,1));

% Construct the matrix of saturation coefficients
% values for two derivatives
t_r2_g =  0:0.1:2;  
t_r2_t = 0:0.1:2; 

figure(1);clf

% for all parameter combinations evaluate the Jacobian

for i = 1:length(t_r2_g),
  for j = 1:length(t_r2_t),
  
       theta = [ 0 0 
	      t_r2_g(i) t_r2_t(j)
	      0 1];
    
    % Jacobian matrix in terms of the normalized variables
    J = LAMBDA*theta; 

    % check determinante and trace;  plot 
       
    if (det(J) == 0)  & (trace(J) ==  0),
      
      	plot(t_r2_g(i),t_r2_t(j),'gx'), hold on     % unstable
           
    elseif det(J) >= 0,
    
      if (trace(J) <= 0) & (trace(J)^2 > 4*det(J)),  
	plot(t_r2_g(i),t_r2_t(j),'+'), hold on    % stab. focus
        
      elseif  (trace(J) <= 0) & (trace(J)^2 < 4*det(J)),  
	
	plot(t_r2_g(i),t_r2_t(j),'r.'), hold on  % stab. eddy
	 	  
      else
	
	plot(t_r2_g(i),t_r2_t(j),'gx'), hold on     % unstable
	
      end
      
    else
      
      	plot(t_r2_g(i),t_r2_t(j),'gx'), hold on     % unstable
	
    end
    
    
  end
end

figure(1)
xlabel('Parameter \theta_a^\rho^2'), ylabel('Parameter \theta_b^\rho^2')
title({'For the previous network, the parameters are varied between 0 and 2';...
'Crosses indicate the instable region while plus signs show the';...
'region where stable nodes can be found. Points mark the parameters range of a stable focus.'})

%________________________________________________________________-
%
% Verification of a selected set of parameters

[k2,k3,K2,KA,J1] = simu_simple(v(1),LAMBDA,X0);

%----------------------------------------------------------------------------
%----------------------------------------------------------------------------

function [k2,k3,K2,KA,J] = simu_simple(c,LAMBDA,X0);

disp('  ')
disp('Simulation of a selected parameter combination:')
disp('  ')
disp('Klick a parameter combination from the Figure')

[th2,th3] = ginput(1);

options = odeset('RelTol',1e-7,'AbsTol',[1e-10],'Vectorized','on');

theta = [ 0 0 
        th2 th3
          0 1];

J = LAMBDA*theta;

% parameters

K2 = th2*X0(1)/(1-th2);
KA = th3*X0(2)^2/(2-th3);

k2 = c/((1-th2)*(1-th3/2));
f1 = X0(1)/(K2+X0(1));
f2 = X0(2)^2/(KA+X0(2)^2);
k3 = 2*c/X0(2);

% time constants to estimate maximal time max_t

ew = eig(J);
max_ew = max(ew);
if real(max_ew) > 0
  max_t = 50;
elseif (imag(max_ew) > 0 & imag(max_ew) > abs(real(max_ew)))
  max_t = ceil(15*imag(max_ew)/(2*pi));
else
  max_t = ceil(8/abs(real(max_ew)));
end

para = [c*1.1;k2;k3;K2;KA;0.1];

time = 0:0.01:max_t;
[t_lin,y_lin] = ode15s(@f_lin,time,[0;0],options,para);
[t_nlin,y_nlin] = ode15s(@f_nonlin,time,X0,options,para);


figure(2);clf;
subplot(211),plot(t_lin,y_lin(:,1)+X0(1),t_lin,y_lin(:,2)+X0(2),'r')
title('Linearized system')

subplot(212);plot(t_nlin,y_nlin(:,1),t_nlin,y_nlin(:,2),'r')
title('Original system')

%%%%




% ----------------------------------------------------------
% ----------------------------------------------------------

function out = f_lin(t,y,para)

in = para(1);
k2 = para(2);
k3 = para(3);
K2 = para(4);
KA = para(5);
delta = para(6);
z1 = y(1,:);
z2 = y(2,:);

X0 = [0.5 0.5]'; % steady state

% Jacobian

J2(1,1) = -k2*X0(2)^2/(X0(2)^2+KA)*K2/(X0(1)+K2)^2;
J2(1,2) = -2*k2*X0(1)/(X0(1)+K2)*X0(2)*KA/(X0(2)^2+KA)^2;
J2(2,1) = 2*k2*X0(2)^2/(X0(2)^2+KA)*K2/(X0(1)+K2)^2;
J2(2,2) = 4*k2*X0(1)/(X0(1)+K2)*X0(2)*KA/(X0(2)^2+KA)^2 - k3;

l = length(z1');
out = J2*[z1;z2]+[ones(1,l)*delta;zeros(1,l)];

% ----------------------------------------------------------
% ----------------------------------------------------------

function out = f_nonlin(t,y,para)

in = para(1);
k2 = para(2);
k3 = para(3);
K2 = para(4);
KA = para(5);

z1 = y(1,:);
z2 = y(2,:);

r2 =  k2*(z1./(K2+z1)).*((z2.^2)./(KA+(z2.^2)));
r3 =  k3*z2; 

out = [in - r2           % G
       2* r2 - r3];      % T
