function  Chapter_9_Figure_7_and_8


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This matlab file reproduces simulation studies in the book:
% Systems Biology: Mathematical modeling and model analysis
% by A. Kremling
%
% It is related to Figure 7 and 8 in Chapter 9
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% extended lactose model open loop
% input: internal lactose (in_lauf)
% output: values after time tmax, plot: values for protein and \Pis
%         after tmax for different values of intern. lactose
% DAE system: one o.d.e. (protein) and one algebraic equation (Dz)


disp(' ')
disp(' Simulation need some time!')
disp(' ')

% --- parameters

V = 2e-15;
N = [-1  1 0 0 0
      1 -1 0 0 0
      0  0 1 -1 1];

L = (abs(N)-N)/2;
L(2,3) = 1;
L(3,1) = 2;

n0 = [2;1;100];
tmax = 1000;
c(1) = 1e-4/6;
c(2) = 1;
c(3) = 3.5;
c(4) = 0.01;
c(5) = 0.2;
c=c';
max_cell = 1000;
pMax = 1200;
mMax = n0(1)+n0(2);
fig_nr=1;


%____________________________________________


NaV = 6.022e23*V;
K = sum(L)';
p.k = c.*((NaV).^(K-1))./(prod(factorial(L))');
p.N = N;
p.L = L;
y0 = [2e-8 0];            % initial values  for forward simulation 
y02 = [9e-7 0];            % initial values for backward simulation

time = 0:tmax/1000:tmax*2; % time
p.LacI_0 = 1e-8;       % overall repressor
p.D_0 = 1e-9;          % overall binding sites
p.K_poly = 1e-1;       % binding Polymerse
p.K_laci = 1e-11;      % binding LacI -> DNA
p.K_l = 2e-9;          % binding lactose -> LacI

p.Kc = 1e-3;           % empirical Crp binding
p.Crp0 = 1e-6;         % Crp concentration


% --- calculation: Figure 9.6

M = [1 0               % Mass matrix for DAE system
     0 0];

options = odeset('RelTol',1e-6,'AbsTol',1e-9,'Vectorized','on','Mass',M);

% loop for internal lactose
in_max = .0025; 
in_lauf = 0:0.000025:in_max;           % interal lactose is input

for l = 1:length(in_lauf);
  
 p.in_lac = in_lauf(l); 
 [t,y] = ode15s(@f1,time,y0,options,p);
 prot(l) = y(end,1);
 dz(l) = y(end,2);
 psi(l) =  dz(l)/p.K_poly/p.D_0;
 psi2(l) = p.in_lac*p.Crp0/(p.in_lac + p.Kc)/1e-6;
 
end

kk = [0.6e3:.5e1:2e3];              % values for closing the loop
 
% --- plotting: Figure 9.6

% Figures shows different values that can be used for closing the loop
% green curve - minimal value only one intersection point
% red curve - maximal value only one intersection point
% black curve - three intersection points

figure(1),subplot(122)
plot(in_lauf,prot,in_lauf,in_lauf/min(kk),'--',in_lauf, ...
		in_lauf/max(kk),'--',in_lauf,in_lauf/1300,'k--'); 
xlim([0 1e-3]),ylim([0 8e-7])
xlabel('lactose (i)'),ylabel('LacY')
title({'stationary characteristics of the open loop S(u) (solid) and the function g^-^1(dashed)'....
    'This yields three intersection points . From the slopes of the points one determines'...
    'that there are two stable and one unstable steady state.'})

figure(1), subplot(121)
plot(in_lauf,psi,in_lauf,psi2), xlabel('lactose (i)'), ylabel('\Psi')
title('stationary characteristics between the lactose input and Do, and between the input and Crp* (active Crp)')

%________________________________________________________________________

% --- calculation: Figure 9.8

for m = 1:length(kk);               % forward simulation

  p.kk = kk(m);
  [t,y] = ode15s(@f2,time,y0,options,p);
  prot_cl(m) = y(end,1);
  lac_in_s(m) = p.kk*prot_cl(m);  

end

for m = 1:length(kk);               % backward simulation

  p.kk = kk(m);
  [t,y] = ode15s(@f2,time,y02,options,p);
  prot_cl_2(m) = y(end,1);

end

% --- plotting: Figure 9.8

figure(2);plot(kk/1000,prot_cl,kk/1000,prot_cl_2,'b--')
xlabel('lactose extern'), ylabel('LacY')
title({'System behavior when the extracellular lactose affects the input and therefore',...
    'the slope of the line g^-^1 is changing. For a small range of the input signal two stable steady states exist.'...
    'The system exhibits hysteresis'})


%___________________________________________________________________________________
%____________________________________________________________________________________

function out = f1(t,y,p)     % open loop

prot = y(1,:);
dz = y(2,:);
psi = dz/p.K_poly/p.D_0;
psi2 = p.in_lac*p.Crp0/(p.in_lac + p.Kc)/1e-6;

out(1,:) = p.k(3)*((psi/2+psi2/2)*2.8e-9) - 0.01*prot;       % dynamic protein
la = p.LacI_0./(1+dz/p.K_laci+p.in_lac.^2/p.K_l);
out(2,:) = p.D_0 - dz.*(1+1/p.K_poly) - dz.*la/p.K_laci;  % algebraic D (closed)


%_________________________________________


function out = f2(t,y,p)     % closed loop

prot = y(1,:);
dz = y(2,:);

lac_in = p.kk*prot;   

psi = dz/p.K_poly/p.D_0;
psi2 = lac_in*p.Crp0/(lac_in + p.Kc)/1e-6;

out(1,:) = p.k(3)*((psi/2+psi2/2)*2.8e-9) - 0.01*prot;       % dynamic protein
la = p.LacI_0./(1+dz/p.K_laci+lac_in.^2/p.K_l);
out(2,:) = p.D_0 - dz.*(1+1/p.K_poly) - dz.*la/p.K_laci;  % algebraic D (closed)
                                                          
